@@ -254,7 +254,7 @@ class MySQLAdapter(BaseAdapter):
 
     def get_field_overrides(self):
         return {
-            'primary_key': 'integer AUTO_INCREMENT',
+            'primary_key': 'INTEGER AUTO_INCREMENT',
             'boolean': 'bool',
             'float': 'float',
             'double': 'double precision',
@@ -603,70 +603,78 @@ class QueryResultWrapper(object):
     - converts rows from the database into model instances
     - ensures that multiple iterations do not result in multiple queries
     """
-    def __init__(self, model, cursor, meta=None):
+    def __init__(self, model, cursor, meta=None, chunk_size=100):
         self.model = model
         self.cursor = cursor
         self.query_meta = meta or {}
         self.column_meta = self.query_meta.get('columns')
         self.join_meta = self.query_meta.get('graph')
+        self.chunk_size = chunk_size
 
         # a query will be considered "simple" if it pulls columns straight
         # from the primary model being queried
         self.simple = self.query_meta.get('simple') or not self.column_meta
 
+        if self.simple:
+            cols = []
+            non_cols = []
+            for i in range(len(self.cursor.description)):
+                col = self.cursor.description[i][0]
+                if col in model._meta.columns:
+                    cols.append((i, model._meta.columns[col]))
+                else:
+                    non_cols.append((i, col))
+            self._cols = cols
+            self._non_cols = non_cols
+            self._iter_fn = self.simple_iter
+        else:
+            self._iter_fn = self.construct_instance
+
         self.__ct = 0
         self.__idx = 0
 
         self._result_cache = []
         self._populated = False
 
-    def model_from_rowset(self, model_class, attr_dict):
-        instance = model_class()
-        for attr, value in attr_dict.iteritems():
-            if attr in instance._meta.columns:
-                field = instance._meta.columns[attr]
+        self.__read_cache = []
+        self.__read_idx = 0
+        self.__read_ct = 0
+
+    def simple_iter(self, row):
+        instance = self.model()
+        for i, f in self._cols:
+            setattr(instance, f.name, f.python_value(row[i]))
+        for i, f in self._non_cols:
+            setattr(instance, f, row[i])
+        return instance
+
+    def construct_instance(self, row):
+        # we have columns, models, and a graph of joins to reconstruct
+        collected_models = {}
+        for i, (model, col) in enumerate(self.column_meta):
+            value = row[i]
+
+            if isinstance(col, tuple):
+                if len(col) == 3:
+                    model = self.model # special-case aggregates
+                    col_name = attr = col[2]
+                else:
+                    col_name, attr = col
+            else:
+                col_name = attr = col
+
+            if model not in collected_models:
+                collected_models[model] = model()
+
+            instance = collected_models[model]
+
+            if col_name in instance._meta.columns:
+                field = instance._meta.columns[col_name]
                 setattr(instance, field.name, field.python_value(value))
             else:
                 setattr(instance, attr, value)
-        return instance
 
-    def _row_to_dict(self, row):
-        return dict((self.cursor.description[i][0], value)
-            for i, value in enumerate(row))
-
-    def construct_instance(self, row):
-        if self.simple:
-            # use attribute names pulled from the result cursor description,
-            # and do not attempt to follow joined models
-            row_dict = self._row_to_dict(row)
-            return self.model_from_rowset(self.model, row_dict)
-        else:
-            # we have columns, models, and a graph of joins to reconstruct
-            collected_models = {}
-            for i, (model, col) in enumerate(self.column_meta):
-                value = row[i]
-
-                if isinstance(col, tuple):
-                    if len(col) == 3:
-                        model = self.model # special-case aggregates
-                        col_name = attr = col[2]
-                    else:
-                        col_name, attr = col
-                else:
-                    col_name = attr = col
-
-                if model not in collected_models:
-                    collected_models[model] = model()
-
-                instance = collected_models[model]
-
-                if col_name in instance._meta.columns:
-                    field = instance._meta.columns[col_name]
-                    setattr(instance, field.name, field.python_value(value))
-                else:
-                    setattr(instance, attr, value)
-
-            return self.follow_joins(self.join_meta, collected_models, self.model)
+        return self.follow_joins(self.join_meta, collected_models, self.model)
 
     def follow_joins(self, joins, collected_models, current):
         inst = collected_models[current]
@@ -691,44 +699,34 @@ class QueryResultWrapper(object):
         return inst
 
     def __iter__(self):
-        self.__idx = 0
+        self.__idx = self.__read_idx = 0
 
         if not self._populated:
             return self
         else:
             return iter(self._result_cache)
 
-    def first(self):
-        try:
-            self.__idx = 0 # move to beginning of the list
-            inst = self.next()
-        except StopIteration:
-            inst = None
-
-        self.__idx = 0
-        return inst
-
-    def fill_cache(self):
-        if not self._populated:
-            idx = self.__idx
-            self.__idx = self.__ct
-            for x in self:
-                pass
-            self.__idx = idx
-
     def iterate(self):
-        row = self.cursor.fetchone()
-        if row:
-            return self.construct_instance(row)
-        else:
-            self._populated = True
-            raise StopIteration
+        if self.__read_idx >= self.__read_ct:
+            rows = self.cursor.fetchmany(self.chunk_size)
+            self.__read_ct = len(rows)
+            if self.__read_ct:
+                self.__read_cache = rows
+                self.__read_idx = 0
+            else:
+                self._populated = True
+                raise StopIteration
+
+        instance = self._iter_fn(self.__read_cache[self.__read_idx])
+        self.__read_idx += 1
+        return instance
 
     def iterator(self):
         while 1:
             yield self.iterate()
 
     def next(self):
+        # check to see if we have a row in our instance cache
         if self.__idx < self.__ct:
             inst = self._result_cache[self.__idx]
             self.__idx += 1
@@ -929,7 +927,7 @@ def parseq(model, *args, **kwargs):
         if isinstance(piece, (Q, R, Node)):
             node.children.append(piece)
         else:
-            raise TypeError('Unknown object: %s', piece)
+            raise TypeError('Unknown object: %s' % piece)
 
     if kwargs:
         node.children.append(Q(model, **kwargs))
@@ -1435,10 +1433,16 @@ class SelectQuery(BaseQuery):
                         ordering = 'ASC'
                 else:
                     raise ValueError('Incorrect arguments passed in order_by clause')
-            else:
+            elif isinstance(clause, basestring):
                 model = self.query_context
                 field = clause
                 ordering = 'ASC'
+            elif isinstance(clause, Field):
+                model = clause.model
+                field = clause.name
+                ordering = 'ASC'
+            else:
+                raise ValueError('Unknown value passed in to order_by')
 
             order_by.append(
                 (model, field, ordering)
@@ -2117,10 +2121,18 @@ class DecimalColumn(Column):
         return {
             'max_digits': 10,
             'decimal_places': 5,
+            'auto_round': False,
+            'rounding': decimal.DefaultContext.rounding,
         }
 
     def db_value(self, value):
-        return value or decimal.Decimal(0)
+        D = decimal.Decimal
+        if not value:
+            return D(0)
+        if self.attributes['auto_round']:
+            exp = D(10)**(-self.attributes['decimal_places'])
+            return D(str(value)).quantize(exp, rounding=self.attributes['rounding'])
+        return value
 
     def python_value(self, value):
         if value is not None:
@@ -2149,6 +2161,10 @@ class FieldDescriptor(object):
     def __set__(self, instance, value):
         setattr(instance, self._cache_name, value)
 
+def qdict(op):
+    def fn(self, rhs):
+        return Q(self.model, **{'%s__%s' % (self.name, op): rhs})
+    return fn
 
 class Field(object):
     column_class = None
@@ -2158,7 +2174,7 @@ class Field(object):
     _order = 0
 
     def __init__(self, null=False, db_index=False, unique=False, verbose_name=None,
-                 help_text=None, db_column=None, default=None, *args, **kwargs):
+                 help_text=None, db_column=None, default=None, choices=None, *args, **kwargs):
         self.null = null
         self.db_index = db_index
         self.unique = unique
@@ -2166,6 +2182,7 @@ class Field(object):
         self.help_text = help_text
         self.db_column = db_column
         self.default = default
+        self.choices = choices
 
         self.attributes = kwargs
 
@@ -2206,6 +2223,21 @@ class Field(object):
     def class_prepared(self):
         pass
 
+    __eq__ = qdict('eq')
+    __ne__ = qdict('ne')
+    __lt__ = qdict('lt')
+    __le__ = qdict('lte')
+    __gt__ = qdict('gt')
+    __ge__ = qdict('gte')
+    __lshift__ = qdict('in')
+    __rshift__ = qdict('isnull')
+    __mul__ = qdict('contains')
+    __pow__ = qdict('icontains')
+    __xor__ = qdict('istartswith')
+
+    def __neg__(self):
+        return (self.model, self.name, 'DESC')
+
 
 class CharField(Field):
     column_class = VarCharColumn
@@ -2416,6 +2448,13 @@ class BaseModelOptions(object):
         self.columns = {}
         self.model_class = model_class
 
+    def prepared(self):
+        # called when _meta is finished being initialized
+        self.defaults = {}
+        for field in self.fields.values():
+            if field.default is not None:
+                self.defaults[field.name] = field.default
+
     def get_sorted_fields(self):
         return sorted(self.fields.items(), key=lambda (k,v): (k == self.pk_name and 1 or 2, v._order))
 
@@ -2537,6 +2576,8 @@ class BaseModel(type):
         for field in _meta.fields.values():
             field.class_prepared()
 
+        _meta.prepared()
+
         if hasattr(cls, '__unicode__'):
             setattr(cls, '__repr__', lambda self: '<%s: %r>' % (
                 _meta.model_name, self.__unicode__()))
@@ -2557,13 +2598,12 @@ class Model(object):
             setattr(self, k, v)
 
     def initialize_defaults(self):
-        for field in self._meta.fields.values():
-            if field.default is not None:
-                if callable(field.default):
-                    field_value = field.default()
-                else:
-                    field_value = field.default
-                setattr(self, field.name, field_value)
+        for field_name, default in self._meta.defaults.items():
+            if callable(default):
+                val = default()
+            else:
+                val = default
+            setattr(self, field_name, val)
 
     def prepared(self):
         # this hook is called when the model has been populated from a db cursor
